GCInfo = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceConsole-2.0")
GCInfo:RegisterDB("GCInfoDB")
GCInfo:RegisterDefaults("profile", {
	Locked = false,
	PosX = nil,
	PosY = nil
})

function GCInfo:OnInitialize()
	self.gnome = AceLibrary("Metrognome-2.0")

	GCInfo:RegisterChatCommand({"/gcinfo", "/gci"}, {
		desc = "Displays GCInfo in a nice bar.",
		type = "group",
		args = {
			lock = {
				name = "Lock",
				desc = "Toggle frame lock.",
				type = "toggle",
				get = function() return self.db.profile.Locked end,
				set = function(val) self.db.profile.Locked = val end,
			}
		}
	})

	self:MakeFrame()
end

-- Create the frame
function GCInfo:MakeFrame()
	GCInfoFrame = CreateFrame("Frame", "GCInfoFrame", UIParent)
	GCInfoFrame:SetFrameStrata("MEDIUM")
	GCInfoFrame:EnableMouse(true)
	GCInfoFrame:SetMovable(true)
	GCInfoFrame:SetWidth(420)
	GCInfoFrame:SetHeight(26)

	GCInfoFrame:SetBackdrop({
		bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", tile = true, tileSize = 16,
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,
		insets = {left = 5, right = 5, top = 5, bottom = 5},
	})

	GCInfoFrame:RegisterForDrag("LeftButton")
	GCInfoFrame:SetScript("OnDragStart", function() if not self.db.profile.Locked then GCInfoFrame:StartMoving() end end)
	GCInfoFrame:SetScript("OnDragStop", function() GCInfoFrame:StopMovingOrSizing() self:SavePosition() end)
	
	local GCInfoText = GCInfoFrame:CreateFontString("GCInfoText", "ARTWORK")
	GCInfoText:SetFontObject(GameFontNormal)
	GCInfoText:SetPoint("CENTER", GCInfoFrame, "CENTER")
	GCInfoText:SetTextColor(1,1,1,1)
end

-- Start
function GCInfo:OnEnable()
	self:RestorePosition()
	self.gnome:RegisterMetro("GCInfo", self.UpdateText, 1, self)
	self.gnome:StartMetro("GCInfo")
	GCInfoFrame:Show()
end

function GCInfo:OnDisable()
	self.gnome:UnregisterMetro("GCInfo")
	GCInfoFrame:Hide()
end

function GCInfo:UpdateGCInfo()
	self.previousMem = self.currentMem
	self.currentMem = collectgarbage("count")
end

function GCInfo:UpdateText()
	self:UpdateGCInfo()
	if (self.previousMem) then
		local str = string.format("Used: [%.1fMB] Increase: [%.1fKB/s]",
				self.currentMem/1024,
				self.currentMem - self.previousMem
		)
		GCInfoText:SetText(str)
	end
end

function GCInfo:RestorePosition()
	local x = self.db.profile.PosX
	local y = self.db.profile.PosY
	if not x or not y then
		GCInfoFrame:ClearAllPoints()
		GCInfoFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
		return
	end

	local s = GCInfoFrame:GetEffectiveScale()
	x,y = x/s, y/s
	
	GCInfoFrame:ClearAllPoints()
	GCInfoFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
end

function GCInfo:SavePosition()
	local x,y = GCInfoFrame:GetLeft(), GCInfoFrame:GetTop()
	local s = GCInfoFrame:GetEffectiveScale()

	x,y = x*s, y*s

	self.db.profile.PosX = x
	self.db.profile.PosY = y
end
